/**
 * FileParser.hpp
 * Author: Yuan Wang
 * Read in an XML file, parse it and build index for it.
 */

#include <util/PlatformUtils.hpp>
#include <internal/URLInputSource.hpp>
#include <parsers/SAXParser.hpp>

#include "Util.h"
#include "FileParserHandler.h"


/**
 * This class is used to parse an XML file and send some elements to indices.
 */
class FileParser
{
public:

  /**
   * Constructor.
   * @param rootName      record root name
   * @param indexName     index names
   * @param indexCount    # of indices
   * @param doValidation  parse with validation
   */
  FileParser ( char* rootName, char* indexName[], char* indexType, int* fieldLength,
	       int indexCount, bool doValidation );

  /**
   * Destructor.
   */
  ~FileParser ();

  /**
   * Parse a file.
   * @param fileName  file name
   */
  void parse ( char* fileName );
	void Run();

private:

  SAXParser* parser;
  FileParserHandler* handler;

  FileStore* fileStore;
  RecordStore* recordStore;

  Sort* sort;
};
