/**
 * FileParserHandler.hpp
 * Author: Yuan Wang
 * Handler for SAX parser
 */

#include <sax/HandlerBase.hpp>

// From Ying Wu
#include "FileStore.h"
#include "RecordStore.h"

// From Wei Shen
#include "sort.h"
#include "searchbtree.h"
#include "filenamelist.h"
#include "dirpool.h"


class FileParserHandler : public HandlerBase
{
public:

  /**
   * Constructor
   */
  FileParserHandler ();

  /**
   * Destructor
   */
  ~FileParserHandler ();


  /**
   * Set Ying Wu and Wei Shen's Objects
   */
  void setStorageObject ( FileStore* fileStore, RecordStore* recordStore, Sort* sort );

  /**
   * Set fid.
   */
  void setFileId ( int fid );

  /**
   * Set names of elements that are being indexed. and open buffer
   */
  void setIndexName ( char* rootName, char* indexNames[], int indexCount, int* fieldLength );

  /**
   * Implementation of SAX document handler interface.
   */
  void endDocument();
  void endElement( const XMLCh* const name );
  void characters( const XMLCh* const chars, const unsigned int length );
  void ignorableWhitespace ( const XMLCh* const chars,
			     const unsigned int length );

  void processingInstruction ( const XMLCh* const target,
			       const XMLCh* const data );

  void startDocument();
  void startElement( const XMLCh* const name, AttributeList& attributes );


  /**
   * Implementation of SAX ErrorHandler interface.
   */
  void warning( const SAXParseException& exception );
  void error( const SAXParseException& exception );
  void fatalError( const SAXParseException& exception );


  /**
   * Implementation of SAX DTDHandler interface.
   */
  void notationDecl ( const XMLCh* const name, const XMLCh* const publicId,
		      const XMLCh* const systemId );

  void unparsedEntityDecl ( const XMLCh* const name, const XMLCh* const publicId,
			    const XMLCh* const systemId, const XMLCh* const notationName );


private:

  char* rootName;
  char* elementContent;
  char* indexNames[2];
  char* indexValue[2][20];
  int indexValueCount[2];
  int fid, indexCount, indexId;
  unsigned short recordCount;
  bool inRecord, doIndex, sameLevel;

  // Objects of Ying Wu and Wei Shen's classes.
  FileStore* fileStore;
  RecordStore* recordStore;
  Sort* sort;

  /**
   * Initialize before reading a record.
   */
  void initRecord ();
};


/**
 * Used to translate charactre code.
 */
class StrX
{
public :

  /**
   * Constructor
   */
  StrX( const XMLCh* const toTranscode, const unsigned int len = 0 ) : fLocalForm( 0 )
    {
      // Call the private transcoding method
      transcode( toTranscode, len );
    }

  /**
   * Destructor
   */
  ~StrX()
    {
      delete [] fLocalForm;
    }

  /**
   * Getter methods
   */
  const char* localForm() const
    {
      return fLocalForm;
    }


private :

  void transcode ( const XMLCh* const toTranscode, const unsigned int len );

  /**
   * fLocalForm
   * This is the local code page form of the string.
   */
  char*   fLocalForm;
};

inline ostream& operator<<( ostream& target, const StrX& toDump )
{
  target << toDump.localForm();
  return target;
}
