#include <stdio.h>
#include <unistd.h>

#include "FileStore.h"
#include "RecordStore.h"

#define MAXFILEID             10000000000


FileStore::FileStore(){
   begin_fid   = 0;
   current_fid = 0;
   pageStore = NULL;
   scan_page = NULL;
}

FileStore::~FileStore(){
   
   if ((scan_page !=NULL)&&(scan_page!=pageStore)) 
     delete scan_page;

   if (pageStore!=NULL) {
     
     pageStore->writeToDisk(getFileName(current_fid));
     delete pageStore;
   } 

}

// return the next new fid
int FileStore::createNewFile(){
       if (pageStore==NULL) // already a new page.
          return current_fid;
	  
       pageStore->writeToDisk(getFileName(current_fid)); // store the old
       delete pageStore;
       current_fid ++;

       pageStore = NULL;
       return current_fid;	
}

bool FileStore::insertRecord(char *record, unsigned short rec_size,
                         int & fid, unsigned short &rid){

	if (pageStore==NULL){
	    pageStore = new PageStore();
	    pageStore->createNewPage();
	    
	    if (pageStore->insertRecord(record,rec_size,rid)){
	        fid = current_fid;
		return true;
	    }else {
	        printf("Warning: this record is larger than page size!");
		return false;
	    }
	}

	if (pageStore->insertRecord(record,rec_size,rid)){
	    fid = current_fid;
	    return true;
	}

	// this page is full, save to next page.
	pageStore->writeToDisk(getFileName(current_fid));
	delete pageStore;
        current_fid ++;
	
	//create new page;
	pageStore = new PageStore();
	pageStore->createNewPage();

	if (pageStore->insertRecord(record,rec_size,rid)){
	    fid = current_fid;
	    return true;
	} else {
            printf("Warning: this record is larger than page size!");
	    return false;	
	}
}

bool FileStore::getRecord(char *&record, unsigned short &rec_size,
                      int fid, unsigned short rid){
	
	/*if (fid== current_fid)
		return pageStore->getRecord(record, rec_size, rid);
	else {
	    PageStore page = PageStore();
	    page.loadFromDisk(getFileName(fid));
	    
	    return page.getRecord(record, rec_size, rid);
	}*/
	return copyRecord(record, rec_size, fid,rid);
	
}

bool FileStore::copyRecord(char *&record, unsigned short &rec_size,
		      int fid, unsigned short rid){

	if (fid== current_fid){
//printf("current file %d\n",fid);
		return pageStore->copyRecord(record, rec_size, rid);
	}else {
	    PageStore page = PageStore();
	    page.loadFromDisk(getFileName(fid));
	    
	    return page.copyRecord(record, rec_size, rid);
	}
	
}		     

bool FileStore::removeFile(int fid){
     
     if (fid== begin_fid) begin_fid++;

     if (fid == current_fid){
           if (pageStore!=NULL) {
	       delete pageStore;
	       pageStore = NULL;
	   }
	   return true;
     }

     if (fid < current_fid){
         if (unlink(getFileName(fid))==0) return true;
	 else return false;
     }

     return false;
}

bool FileStore::getOldID(int fid, int rid, int & yuanFid, int &yuanEid){
     
     char *rec;
     unsigned short rec_size;
     
     if (getRecord(rec, rec_size, fid, rid)){
        RecordStore * recordStore = new RecordStore();
	yuanFid = 	recordStore->getFid(rec,rec_size);
	yuanEid = 	recordStore->getEid(rec,rec_size);

	return true;
     }
     else return false;
}

int FileStore::beginScan(char* &record, unsigned short & size,
                         int & fid, unsigned short & rid){
     if (scan_page!=NULL) {
//printf("scan_page is not null\n");
//getchar();
       delete scan_page;
     }
     scan_page =NULL;

     if (begin_fid == current_fid) {
         if (pageStore == NULL) return -1; //empty in system.

	 scan_fid = current_fid; //only page is memory
	 scan_rid = 0;
         
	 if (pageStore->getRecord(record,size,0)) { 
	    
	    fid = scan_fid;
	    rid = scan_rid;
	    scan_page = pageStore;
 
	    return 1;
         }
	 return -1; // no record in current;
     }

     scan_fid = begin_fid;
     scan_rid = 0;

     scan_page = new PageStore();
     scan_page->loadFromDisk(getFileName(begin_fid));

     if (scan_page->getRecord(record,size,0)){
         
	 fid = scan_fid;
	 rid = scan_rid;
         
	 return 1;
     }else {
          printf("Wrong: %d file is empty\n",begin_fid);
	  return -1;
     }
     
}

int FileStore::getNext(char * &record, unsigned short & size,
                       int & fid, unsigned short & rid){
     
//printf("%d %d ",scan_fid, scan_rid);
     
     scan_rid ++;

     if (scan_page->getRecord(record,size,scan_rid)) {
         
	 fid = scan_fid;
	 rid = scan_rid;

	 return 1;
     }
     // no more record in current page
     if (current_fid!=scan_fid) {
	delete scan_page;
     }
     
     scan_page =NULL;
//printf("set scan_page to NULL : %d\n",scan_fid);	
     
     scan_rid =0;
     scan_fid ++;

     if (scan_fid > current_fid) return -1;

//printf("\n");

     if (scan_fid == current_fid) {
           if (pageStore == NULL) return -1;  // the end, current file is empty.
	   if (pageStore->getRecord(record,size,0)) {
	      
	      scan_page = pageStore;
	      
	      fid = scan_fid;
	      rid = scan_rid;

	      return 1;
           }
	   return -1; // no record in current.
     }
     
     scan_page = new PageStore();
     scan_page->loadFromDisk(getFileName(scan_fid));

//printf("new scan_page %d\n",scan_fid);

     if (scan_page->getRecord(record,size,0)) {
          fid = scan_fid;
	  rid = scan_rid;

	  return 1;
     }else {
          printf("Wrong: %d file is empty\n",begin_fid);
	  return -1;
     }
     
}
/////////////////////////////////////////////////////////////////
// private function
/////////////////////////////////////////////////////////////////

char * FileStore::getFileName(int fid){

    fileName[0]='f';
    sprintf(fileName+1,"%d",fid);
    return fileName;
}

/*int main(){

     char a[300];
     for (int i=0;i<300;i++){
         if (i<298) a[i] = 'a'; 
	 else if (i==298) a[i]= 'b'; 
	 else a[i]='\0';
     }
     
     bool t;
     int fid;
     unsigned short rid;
     
     FileStore fStore= FileStore();
     for (int i=0;i<100;i++){
          t = fStore.insertRecord(a,300,fid,rid);
	  if (t) printf("write %d %d\n",fid,rid);
	  else printf("write failed\n");
     }
     
     getchar();
     if (fStore.removeFile(0)) printf("remove 0 success!\n");
     else printf("remove failed\n");
     if (fStore.removeFile(3)) printf("remove 3 success!\n");
     else printf("remove failed\n");
     if (fStore.removeFile(4)) printf("remove 0 success!\n");
     else printf("remove failed\n");
     if (fStore.removeFile(0)) printf("remove 0 success!\n");
     else printf("remove failed\n");

     return 0;
}*/
