#include <stdio.h>
#include <math.h>
#include <string.h>

#include "IDTable.h"

IDTable::IDTable(unsigned short ID_bucket_rate =10, 
                 unsigned short Name_bucket_rate = 2){
    
    IDCounter =0;
    
    int ID_Bucket=1;
    int Name_Bucket=1;
    for(int i=0;i<ID_bucket_rate;i++) ID_Bucket*=2;
    for(int i=0;i<Name_bucket_rate;i++) Name_Bucket *=27;

    ID_Table 	= new IDRecord[ID_Bucket];
    for (int i=0;i<ID_Bucket;i++){
       ID_Table[i].ID  =INVALID;
       ID_Table[i].next=NULL;
    }

    Name_Table	= new NameRecord[Name_Bucket];
    for (int i=0;i<Name_Bucket;i++){
       for (int j=0;j<NAMESIZE;j++) 
         Name_Table[i].ID[j]= INVALID;
         Name_Table[i].next = NULL;
    }

    id_rate   = ID_Bucket;
    name_rate = Name_bucket_rate;

}

IDTable::~IDTable(){
   delete [] ID_Table;
   delete [] Name_Table;
}

unsigned short IDTable::insertElement(char *name,char type = 'e'){
    
    unsigned short id;

    // alreay in IDTable.
    if (IDCounter>0) if (getID(name,id)) return id;
    
    //printf("new id\n"); 
    
    // insert ID_Table;
    if (ID_Table[IDCounter%id_rate].ID==INVALID){
       ID_Table[IDCounter%id_rate].ID= IDCounter;
       if (strlen(name) <=NAMELENGTH){
          strcpy(ID_Table[IDCounter%id_rate].name, name);
       }
       else {
         memcpy(ID_Table[IDCounter%id_rate].name, name, NAMELENGTH);
       }
       ID_Table[IDCounter%id_rate].type=type;
       
       //insertInNameTable(name,IDCounter);
       IDCounter ++;
       return (IDCounter-1);
    }

    //printf("IDCounter, %d, id_rate %d\n",IDCounter, id_rate);
    //printf("id value %d\n",ID_Table[IDCounter%id_rate].ID);

    IDRecord * record = new IDRecord;
    record->ID = IDCounter;
    if (strlen(name) <=NAMELENGTH){
       memcpy(record->name, name, strlen(name));
    }
    else {
       memcpy(record->name, name, NAMELENGTH);
    }
    record->type=type;
    record->next= ID_Table[IDCounter%id_rate].next;
    ID_Table[IDCounter%id_rate].next = record;

    insertInNameTable(name,IDCounter); 
    IDCounter++;

    return (IDCounter-1);
}

bool IDTable::getName(unsigned short id, char * name){
   
   if (ID_Table[id%id_rate].ID==id){
      //name=new char [32];
      int length =strlen(ID_Table[id%id_rate].name);
      if (length <NAMELENGTH) 
         strcpy(name,ID_Table[id%id_rate].name);
      else memcpy(name,ID_Table[id%id_rate].name,NAMELENGTH);

      return true;
   }
   
   IDRecord * record=(IDRecord *)(ID_Table[id%id_rate].next);
   while(record!=NULL){
      if(record->ID==id){
         //name=new char [32];
	 memcpy(name,record->name,NAMELENGTH);
	 return true;
      }
      else record=(IDRecord *)(record->next);
   }

   return false;
}

bool IDTable::getID(char *name, unsigned short &id){
    
    if (IDCounter <= SIMPLE) return simple_getID(name, id);
    
    int index = getNameBucketEntry(name);
    NameRecord * nameRecord = &(Name_Table[index]);
    char * cmpname = new char [NAMELENGTH];
    
    while (nameRecord!=NULL){
      for(int i=0;i<NAMESIZE;i++){
         getName(nameRecord->ID[i],cmpname);
	 if (strcmp(cmpname,name)==0){ 
	    delete [] cmpname;
	    return nameRecord->ID[i];
	 } 
      }
      nameRecord = (NameRecord *)(nameRecord->next);
    }
    return false;
}

bool IDTable::simple_getID(char * name, unsigned short & id){
    
    char * compare = new char [NAMELENGTH];
    	
    for (int i=0;i<IDCounter;i++){
       getName(i, compare);
       //printf("%d,%s and %s\n",i,name, compare); 
       if (strcmp(compare, name)==0) {
          delete [] compare;
	  //printf("equal, return %d\n",i);
	  id = i;
	  return true;
       }
    }
    
    delete [] compare;
    return false;
}

int IDTable::getNameBucketEntry(char *name){
    
    int index[name_rate];
    int len= strlen(name);
    
    for (int i=0;i<name_rate;i++){
       if (i < len){
          if ((name[i]>='a')&&(name[i]<='z')) index[i]=name[i]-'a';
	  else if ((name[i]>='A')&&(name[i]<='Z')) index[i]=name[i]-'A';
	  else index[i]=26;
       }
       else index[i]=26;
    }

    int index_name = 0;
    for (int i=0;i<name_rate;i++){
       index_name *=27;
       index_name+=index[i];
    }

    return index_name;
}

void IDTable::insertInNameTable(char *name, unsigned short id){
   
    int index = getNameBucketEntry(name);

    NameRecord * nameRecord  = &(Name_Table[index]);
    NameRecord * father;
    while(nameRecord!=NULL){
      for (int i=0;i<NAMESIZE;i++){
          if (nameRecord->ID[i]==INVALID){
	     nameRecord->ID[i]=id;
	     return;
	  }   
      }
      father    = nameRecord; 
      nameRecord= (NameRecord *)(nameRecord->next);
    }
    
    nameRecord= new NameRecord;
    nameRecord->ID[0]=id;
    for (int i=1;i<NAMESIZE;i++) nameRecord->ID[i]=INVALID;
    nameRecord->next =NULL;
    
    father->next = nameRecord;
}

void IDTable::dump(){
    
    int j;
    IDRecord * record;
    printf("================dump=======================\n");
    printf("%d name stored\n", IDCounter);
    printf("\nIDtable:\n");
    for(int i=0;i<id_rate;i++){
        j=i;
	while(j<IDCounter){
             if (j==i){
	       printf("%d :%d%c %15s ",j,ID_Table[i].ID, 
	               ID_Table[i].type,ID_Table[i].name);
	       record=(IDRecord *)(ID_Table[i].next);
	       if (record==NULL) {
	         printf("\n");
		 break;
	       }else j=record->ID;	 
             }		       
	     else {
		j=record->ID;
		printf("%d%c %15s ",j,record->type,record->name);

		record= (IDRecord *)(record->next);
		if (record==NULL) {
		   printf("\n");
		   break;
		}   
	     }
	}
    }
    
    printf("\nNameTable:\n");
    bool t;
    int max =(int)exp(name_rate*log(27));
    
    for(int i=0;i<max;i++){
       NameRecord * record = &(Name_Table[i]);
       t=false;
       while (record !=NULL){
          for (int j=0;j<NAMESIZE;j++){
              if (record->ID[j]!=INVALID) {
	         printf("%d %d",record->ID[j],j);
	         t= true;
	      }
          }
	  record= (NameRecord *)(record->next);
       }
       int k=i; 
       if(t) {
           while(i>0) {
	     printf("%c",i%27+'a');
	     i/=27;
	   }  
	   printf("\n");
       }
       i=k;
    }

    
}

/*main(){
   bool t;
   IDTable idT = IDTable(2,1);

   unsigned short id = idT.insertElement("Books");
   idT.insertElement("Authors");
   idT.insertElement("Date");
   idT.insertElement("Paper");
   idT.insertElement("lastName");
   idT.insertElement("firstName");
   idT.insertElement("publisher");

   t= idT.getID("hello",id);
   if (t) printf("true\n"); else printf("false\n");
   
   t=idT.getID("Date", id);
   printf("id is %d\n",id);

   char * name =  new char [32];
   idT.getName(3,name);
   printf("name is %s\n",name);

   idT.dump();
   
}*/
