#include <stdio.h>
#include <stdlib.h>

#include <unistd.h>

#include "PageStore.h"

int PageStore::count       = 0;
int PageStore::pagesize    = 4*getpagesize();
int PageStore::slotSize    = 2*sizeof(unsigned short);
int PageStore::reserveSize = 10;
long PageStore::readtime    = 0;
long PageStore::writetime    = 0;
long PageStore::opentime    = 0;
long PageStore::closetime    = 0;

void PageStore::initTime(){
     opentime=0;
     closetime=0;
     readtime=0;
     writetime=0;
}

void PageStore::showTime(){
    printf("In storage :\n open+close %ld read+write %ld total %ld\n",opentime+closetime, readtime+writetime, opentime+closetime+readtime+writetime);
}

PageStore::PageStore(char *mem_page=NULL){
   // increment counter
   count ++;

   // create the memory
   if (mem_page==NULL) {
      mem = new char[pagesize];
      for (int i=0;i<pagesize;i++) mem[i]=(char)0;
   }else mem=mem_page;
}

PageStore::~PageStore(){
   delete []mem ;
}

int PageStore::getPageSize(){
   //printf("pagesize is %d\n",pagesize);
   return pagesize;
}

int PageStore::getPageCounter(){
    return count;
}

void PageStore::createNewPage(){
    // initialize 
    recordCounter = 0;
    emptySpace = pagesize-reserveSize;
}

bool PageStore::insertRecord(char *record, unsigned short rec_size, 
                             unsigned short &rid){
    
    //check if it has enough space.
    if (emptySpace < slotSize + rec_size) return false;
    
    unsigned short location;
    int slot_loc;
    
    //find the space.
    if (recordCounter==0) {
      location = reserveSize;
      slot_loc = pagesize;
    }
    else {
         slot_loc=(pagesize-recordCounter*slotSize);
	 
	 unsigned short temp;
	 //location = pointer+size;
	 memcpy((char*)(&location), mem+slot_loc, sizeof(location));
	 memcpy((char*)(&temp), mem+slot_loc+sizeof(temp), sizeof(temp));
	 location+=temp;
    }
    
    //printf("%d \n",slotSize);
    
    // create new slot copy address and size into slot. 
    memcpy(mem+slot_loc-slotSize,(char *)(&location), sizeof(location));
    memcpy(mem+slot_loc-slotSize+sizeof(location),
           (char *)(&rec_size), sizeof(rec_size));

    // copy the data into memory.
    memcpy(mem+location, record, rec_size);
    
    // modify 
    rid = recordCounter;
    recordCounter ++;
    emptySpace -= (slotSize + rec_size);
    
    return true;
}

bool PageStore::getRecord(char *&record, unsigned short &rec_size, unsigned short rec_id){
    

    return copyRecord(record, rec_size, rec_id);
    //test if such record exist.
    /*if (recordCounter <= rec_id) return false;
    
    //get record location and size.
    unsigned short temp;
    memcpy((char *)&temp, mem+pagesize-(rec_id+1)*slotSize, sizeof(temp));
    record = mem+ (int) temp;
    memcpy((char *)&temp, mem+pagesize-(rec_id+1)*slotSize+sizeof(temp),
           sizeof(temp));
    rec_size = temp;

    return true;*/
} 

bool PageStore::copyRecord(char *&record, unsigned short &rec_size, unsigned short rec_id){
    
    //printf("counter: %d id: %d\n",recordCounter, rec_id);

    //test if such record exist.
    if (recordCounter <= rec_id) return false;
    
    //get record location and size.
    unsigned short temp;
    memcpy((char *)&temp, mem+pagesize-(rec_id+1)*slotSize+sizeof(temp),
           sizeof(temp));
    rec_size = temp;
    
    //allocate new memory for record.
    record   = new char [rec_size];
    
    // copy to destionation
    memcpy((char *)&temp, mem+pagesize-(rec_id+1)*slotSize,sizeof(temp));
    memcpy(record, mem+(int)temp, rec_size);
    
// commented by weishen
//    printf("size is in pagestore:%d\n",rec_size);

    return true;
} 

bool PageStore::writeToDisk(char *filename){
    
    memcpy(mem, (char *)(&emptySpace), sizeof(emptySpace));
    memcpy(mem+sizeof(emptySpace), 
           (char *)(&recordCounter), sizeof(recordCounter));
    
    FILE *fp;
    
    gettimeofday(&tval, 0);
    if((fp=fopen(filename,"w"))){
       gettimeofday(&tval1,0);
       PageStore::opentime += getDiff(); 

       gettimeofday(&tval, 0);
       int size_t = fwrite(mem,1,pagesize,fp);
       gettimeofday(&tval1,0);
       PageStore::writetime += getDiff();
       
       gettimeofday(&tval,0);
       fclose(fp);
       gettimeofday(&tval1,0);
       PageStore::closetime += getDiff();

       if (size_t!=pagesize) return false;
       else return true;
    }
    else return false;
}

bool PageStore::loadFromDisk(char *filename){
    FILE *fp;
    
    gettimeofday(&tval,0);
    if((fp=fopen(filename,"r"))){
       gettimeofday(&tval1,0);
       PageStore::opentime += getDiff(); 
       
       gettimeofday(&tval,0);
       int size_t = fread(mem,1,pagesize,fp);
       gettimeofday(&tval1,0);
       PageStore::readtime += getDiff(); 
       
       gettimeofday(&tval,0);
       fclose(fp);
       gettimeofday(&tval1,0);
       PageStore::closetime += getDiff();

       if (size_t!=pagesize) return false;
    
    }
    else return false;

    memcpy((char *)(&emptySpace), mem, sizeof(emptySpace));
    memcpy((char *)(&recordCounter),
           mem+sizeof(emptySpace), sizeof(recordCounter));
	   
    return true;
}

void PageStore::dumpPage(){
   
   for (int i=0;i<=1+pagesize/20;i++){
     printf("%d: ",i);
     for (int j=0;j<20;j++)
       if (i*20+j<pagesize) printf("%d ",mem[i*20+j]);
     printf("\n");  
   }

}

long PageStore::getDiff(){
     long temp;

     temp = (tval1.tv_sec-tval.tv_sec)*1000000;
     temp +=(tval1.tv_usec-tval.tv_usec);
     return temp;
}
/*main(){
   PageStore ps=PageStore();
   
   char *record = new char[26];
   for (int i=0;i<26;i++) record[i]='a'+i;
   char *rec;
   unsigned short size;
   
   ps.createNewPage();
   ps.insertRecord(record, 26);
   ps.insertRecord(record, 26);
   ps.insertRecord(record, 20);
   ps.writeToDisk("page1");

   PageStore ps1=PageStore();
   bool t=ps1.loadFromDisk("page1");
   //if (t) printf("true\n");else printf("false\n");
   if (ps1.getRecord(rec,size,2)){
      printf("get record of size %d\n",size);
      for (int i=0;i<size;i++)
        printf("%c",rec[i]);
      printf("\n");	
   }
   else printf("get record failed\n");
   //if (t) printf("true\n"); else printf("false\n");
   
   
}*/
